package com.icontrols.kakao.controller;

import java.util.Base64;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.UUID;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.service.AuthService;
import com.icontrols.kakao.service.KakaoExtensionService;
import com.icontrols.kakao.socket.wallpad.KakaoMsgHandler;

@RestController
public class MainController {

	private static final Logger logger = LoggerFactory.getLogger(MainController.class);
	
	@Resource( name = "kakaoMsgHandler")
	KakaoMsgHandler kakaoMsgHandler;
	
	@Autowired
	AuthService authService;
	
	@Autowired
	KakaoExtensionService service;
	
	@RequestMapping(value = "/test", method = RequestMethod.POST, consumes = "application/json")
	public @ResponseBody Map<String, Object> recieveTest(@RequestHeader HttpHeaders header,
														@RequestBody Map<String, Object> params,
														HttpServletResponse response, 
														HttpServletRequest request) throws Exception {
		
		logger.debug("kakaoMsgHandler is null" + kakaoMsgHandler);
		logger.info("===================================================================================");
		logger.info("==========================================이벤트 수신=================================");
		logger.info("===================================================================================");
		logger.info("body :" + params.toString());

		Map<String,Object> retMap = new HashMap<String, Object>();
		
		retMap.put(KEY_INFO.HTTP_KEY_ID, params.get(KEY_INFO.HTTP_KEY_ID));
		retMap.put(KEY_INFO.HTTP_KEY_TYPE, params.get(KEY_INFO.HTTP_KEY_TYPE));
		retMap.put(KEY_INFO.HTTP_KEY_VERSION, params.get(KEY_INFO.HTTP_KEY_VERSION));
		
		String jsonString = "{\"name\":\"mkyong\", \"age\":29}";
		Map<String, Object> testmap = CommonUtil.jsonToMap(jsonString);
		
		logger.debug("--------->>>>>  "+testmap.get("name").toString());

		retMap.putAll(KakaoMsgHandler.getDeviceCapabilities("light"));
		
		String originAuth = SERVERINFO.KAKAO_EVENT_CLIENT_ID+":"+SERVERINFO.KAKAO_EVENT_CLIENT_SECRET;
		String encode = Base64.getEncoder().encodeToString(originAuth.getBytes());
		
		logger.debug("____________" + encode);
		
		String decode = new String(Base64.getDecoder().decode(encode));
		logger.debug("decode --> " + decode);
		
		originAuth = SERVERINFO.KAKAO_HEADER_CLIENT_ID+":"+SERVERINFO.KAKAO_HEADER_CLIENT_SECRET;
		encode = Base64.getEncoder().encodeToString(originAuth.getBytes());
		
		logger.debug("_____HEADER_______" + encode);
		
		decode = new String(Base64.getDecoder().decode(encode));
		logger.debug("decode --> " + decode);
		
		
		return retMap;
	}
	
	@RequestMapping(value = "/events", method = RequestMethod.POST, consumes = "application/json")
	public @ResponseBody Map<String, Object> recieveEvents(@RequestHeader HttpHeaders header,
														@RequestBody Map<String, Object> params) throws Exception {
		
		logger.info("===================================================================================");
		logger.info("==========================================이벤트 수신=================================");
		logger.info("===================================================================================");
		logger.info("header" + header.toString());
		logger.info("header access = " + header.getOrigin());
		logger.info("body :" + params.toString());

		Map<String,Object> retMap = new HashMap<String, Object>();
		
		retMap.put(KEY_INFO.HTTP_KEY_ID, params.get(KEY_INFO.HTTP_KEY_ID));
		retMap.put(KEY_INFO.HTTP_KEY_TYPE, params.get(KEY_INFO.HTTP_KEY_TYPE));
		retMap.put(KEY_INFO.HTTP_KEY_VERSION, params.get(KEY_INFO.HTTP_KEY_VERSION));
		
		Random rd = new Random();
		
		int rdNum = rd.nextInt(10);
		if(rdNum % 2 == 0) {
			rdNum = rd.nextInt(20);
			if(rdNum > 10)
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_EXPIREDTOKEN));
			else
				retMap.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_TOKEN));
		}
		return retMap;
	}
	
	@RequestMapping(value = "/events/token", method = RequestMethod.POST, consumes = "application/x-www-form-urlencoded")
	public @ResponseBody Map<String, Object> recieveEventsToken(@RequestHeader HttpHeaders header,
														@RequestBody(required = false) String grant_type,
														@RequestBody(required = false) String refresh_token) throws Exception {
		
		logger.info("===================================================================================");
		logger.info("==========================================토큰 재생성=================================");
		logger.info("===================================================================================");
		logger.info("header" + header.toString());
		logger.info("header access = " + header.getOrigin());
		logger.info("granttype = " + grant_type);
		logger.info("refresh token = " + refresh_token);

		Map<String,Object> retMap = new HashMap<String, Object>();
		
		retMap.put("access_token", UUID.randomUUID().toString());
		retMap.put("refresh_token", UUID.randomUUID().toString());
		retMap.put("expires_in", 3600);
		retMap.put("token_type", "bearer");
		
		return retMap;
	}
	
	@RequestMapping(value = "/subs", method = RequestMethod.POST, consumes = "application/json")
	public @ResponseBody Map<String, Object> subscription(@RequestHeader HttpHeaders header,
														@RequestBody Map<String, Object> params) throws Exception {

		logger.info("===================================================================================");
		logger.info("================================기-기-구-독-/-해-제===================================");
		logger.info("===================================================================================");
		logger.info("body :" + params.toString());

		/*
		 *  type -> subs/unSubs
		 *  dong -> 802
		 *  ho   -> 1103
		 *  company -> kakao 
		 *  int dong = Integer.parseInt((String)param.get("dong"));
			int ho = Integer.parseInt((String)param.get("ho"));
			
			String type = (String) param.get("type");
			String company = (String) param.get("company");
			
			int devNum = Integer.parseInt((String)param.get("devNum"));
		 */
		Map<String,Object> retMap = new HashMap<String, Object>();
		authService.subscriptionProcess(params);
		
		retMap.put(KEY_INFO.HTTP_KEY_ID, params.get(KEY_INFO.HTTP_KEY_ID));
		retMap.put(KEY_INFO.HTTP_KEY_TYPE, params.get(KEY_INFO.HTTP_KEY_TYPE));
		retMap.put(KEY_INFO.HTTP_KEY_VERSION, params.get(KEY_INFO.HTTP_KEY_VERSION));
		
		return retMap;
	}
}
